require 'simplecov'
require 'simplecov-rcov'

SimpleCov.formatter = SimpleCov::Formatter::RcovFormatter
SimpleCov.start do
  add_filter "/spec/"
end

$LOAD_PATH.unshift File.expand_path('../../lib', __FILE__)
$LOAD_PATH.unshift File.expand_path('../../spec/support', __FILE__)

require 'soar_authentication_token'
require 'pry'
require 'auth_token_store_provider'

def one_year_in_seconds
  31536000
end

def request_information_from_valid_source
  { 'source_address'    => '1.1.1.1',
    'user_agent'        => 'some shiny browser',
    'service'           => 'test-service',
    'resource'          => '/'
  }
end

module AuthTokenStoreProvider
  class LocalStubClient
    def store
      @@store ||= []
    end

    def clear_store
      @@store = []
    end

    def initialize(configuration = {})
      if configuration['stub-data'] != {} || configuration['stub-data'] != nil
        @@store = configuration['stub-data']
      end
    end

    def store_failure(state = true)
      @@store_failure = state
    end

    def store_failure?
      @@store_failure ||= false
      @@store_failure
    end

    def add(token_identifier:, authenticated_identifier:, token_issue_time:, token_expiry_time:, flow_identifier: nil)
      raise RuntimeError, 'Failure accessing store' if store_failure?
      store << { token_identifier: token_identifier,
                  authenticated_identifier: authenticated_identifier,
                  token_issue_time: token_issue_time,
                  token_expiry_time: token_expiry_time }
    end

    def remove(token_identifier:, flow_identifier: nil)
      raise RuntimeError, 'Failure accessing store' if store_failure?
      store.delete_if { |x| x[:token_identifier] == token_identifier }
      true
    end

    def token_exist?(token_identifier:, authenticated_identifier:, token_issue_time:, token_expiry_time:, flow_identifier: nil)
      raise RuntimeError, 'Failure accessing store' if store_failure?

      store.each { |x|
        if ((x[:token_identifier] == token_identifier) and
            (x[:authenticated_identifier] == authenticated_identifier) and
            (x[:token_issue_time] == token_issue_time) and
            (x[:token_expiry_time] == token_expiry_time))
          return true
        end
      }
      false
    end

    def remove_tokens_for(authenticated_identifier:, flow_identifier: nil)
      raise RuntimeError, 'Failure accessing store' if store_failure?
      store.delete_if { |x| x[:authenticated_identifier] == authenticated_identifier }
      true
    end

    def list_tokens_for(authenticated_identifier:, flow_identifier: nil)
      raise RuntimeError, 'Failure accessing store' if store_failure?
      store.select { |x| x[:authenticated_identifier] == authenticated_identifier}
    end
  end
end
